local WorldQuestGroupFinderAddon = CreateFrame("Frame", "WorldQuestGroupFinderAddon", UIParent)
local _, L = ...;

WorldQuestGroupFinder = {}
WorldQuestGroupFinderConfig = {}

local DefaultConfig = {}
local RegisteredEvents = {}
WorldQuestGroupFinderAddon:SetScript("OnEvent", function (self, event, ...) if (RegisteredEvents[event]) then return RegisteredEvents[event](self, event, ...) end end)

local currentWQ = nil
local tempWQ = nil

local activityIDs = {
	["419"] = "Aszuna",
	["420"] = "Val'sharah", 
	["421"] = "Highmountain",
	["422"] = "Stormheim", 
	["423"] = "Suramar"
}

local pendingApplications = {}
local blacklistedLeaders = {}

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "WorldQuestGroupFinder") then
		SLASH_WQGF1 = '/wqgf'
		SlashCmdList["WQGF"] = function (msg, editbox)
			WorldQuestGroupFinder.handleCMD(msg, editbox)	
		end
		setmetatable(WorldQuestGroupFinderConfig, {__index = DefaultConfig})
		print("|c00bfffffWorldQuestGroupFinder v0.1 ALPHA. "..L["Click on a World Quest in the objective tracking window to search for a group."])
	end
end

function RegisteredEvents:PLAYER_LOGIN(event)
	WorldQuestGroupFinder.SetHooks()
	WorldQuestGroupFinder.CreateConfigMenu()
end

function RegisteredEvents:QUEST_TURNED_IN(event, questID, experience, money)
	if (currentWQ ~= nil and questID == currentWQ) then
		if (WorldQuestGroupFinder.GetConfigValue("notifyParty") and IsInGroup()) then
			WorldQuestGroupFinder.SendWQCompletionPartyNotification(questID)
		end
		if (WorldQuestGroupFinder.GetConfigValue("askToLeave")) then
			WorldQuestGroupFinder.ShowLeavePrompt()
		end
		currentWQ = nil
	end
end

function RegisteredEvents:LFG_LIST_APPLICATION_STATUS_UPDATED(event, applicationID, status)
	local _,_,name,description,_,ilvl,_,_,_,_,_,_,author,members = C_LFGList.GetSearchResultInfo(applicationID)
	if (status == "applied") then
		pendingApplications[applicationID] = tempWQ
	end
	if (status == "declined" or status == "failed" or status == "invitedeclined" or status == "timedout") then
		blacklistedLeaders[author] = true
		-- If all applications have been declined, restart process. A new group will be created if nothing else is found.
		if (C_LFGList.GetNumApplications() - 1 == 0) then
			WorldQuestGroupFinder.InitWQGroup(pendingApplications[applicationID])
		end
	end
	if (status == "cancelled") then
		-- placeholder
	end
	if (status == "inviteaccepted") then
		WorldQuestGroupFinder.prefixedPrint(string.format(L["You have joined |c00bfffff%s|c00ffffff's group for |c00bfffff%s|c00ffffff. Have fun!"], author, WorldQuestGroupFinder.GetQuestInfo(pendingApplications[applicationID])))
		currentWQ = pendingApplications[applicationID]
		local pendingApplications = {}
		local blacklistedLeaders = {}
	end
end

function RegisteredEvents:LFG_LIST_ENTRY_EXPIRED_TIMEOUT(event)
	print("LFG_LIST_ENTRY_EXPIRED_TIMEOUT")
end

function RegisteredEvents:GROUP_ROSTER_UPDATE(event)
	-- Leaving the group. TRIGGERED TWICE
	if (IsInGroup() == false) then
		currentWQ = nil
	end
end

for k, v in pairs(RegisteredEvents) do
	WorldQuestGroupFinderAddon:RegisterEvent(k)
end

function WorldQuestGroupFinder.GetQuestInfo (questID)
	local tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = GetQuestTagInfo (questID)
	local title, factionID = C_TaskQuest.GetQuestInfoByQuestID (questID)
	return title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex
end

function WorldQuestGroupFinder.SetHooks()	
	local bkp_BonusObjectiveTracker_OnBlockClick = BonusObjectiveTracker_OnBlockClick
	function BonusObjectiveTracker_OnBlockClick(self, button)	
		if (button == "LeftButton") then
			tempWQ = self.TrackedQuest.questID
			if (currentWQ ~= nil and tempWQ ~= currentWQ) then
				StaticPopup_Show ("NEW_WORLD_QUEST_PROMPT")
			else 
				-- No current WQ. 
				if (currentWQ == nil) then
					WorldQuestGroupFinder.InitWQGroup(tempWQ)
				else
					WorldQuestGroupFinder.prefixedPrint(L["You already are in a group for this World Quest."])
				end
			end
		end
		bkp_BonusObjectiveTracker_OnBlockClick(self, button)
	end
end

function WorldQuestGroupFinder.InitWQGroup(wqID) 
	local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(wqID)
	local foundZone = false
	local foundGroup = false
	local currentZone = nil
	-- Finding current zone activity ID
	for k,v in pairs( activityIDs ) do
		if (C_LFGList.GetActivityInfoExpensive(k)) then
			currentZone = k
			foundZone = true
		end
	end
	if (foundZone) then
		-- Clear existing applications
		if (C_LFGList.GetNumApplications() > 0) then
			for k,v in pairs( C_LFGList.GetApplications() ) do
				C_LFGList.CancelApplication(v)
			end
		end
		WorldQuestGroupFinder.prefixedPrint(string.format(L["Searching for a group for World Quest |c00bfffff%s|c00ffffff..."], title))
		-- Search for existing groups
		C_LFGList.ClearSearchResults()
		C_LFGList.Search(1, "#WQ:"..wqID.."#", 0, 4, C_LFGList.GetLanguageSearchFilter());
		-- Delay to let the search finish
		C_Timer.After(2, function()
			local applicationsCount = 0
			local blacklistedApplicationsCount = 0
			local searchCount, searchResults = C_LFGList.GetSearchResults()
			local currentPlayers = 1;
			if (IsInGroup()) then
				currentPlayers = GetNumGroupMembers()
			end
			for k, v in pairs( searchResults ) do
				local _,_,name,description,_,ilvl,_,_,_,_,_,_,author,members = C_LFGList.GetSearchResultInfo(v)
				-- Double check for description content, required ilvl and group members
				if (string.find(description, "#WQ:"..wqID.."#") ~= nil and GetAverageItemLevel() > ilvl and members + currentPlayers <= 5) then
					if (blacklistedLeaders[author] == true) then
						blacklistedApplicationsCount = blacklistedApplicationsCount + 1
					else 
						foundGroup = true;
						local canBeTank, canBeHealer, canBeDamager = UnitGetAvailableRoles("player")
						C_LFGList.ApplyToGroup(v, "WorldQuestGroupFinder User", canBeTank, canBeHealer, canBeDamager)
						applicationsCount = applicationsCount + 1
					end
				end
			end
			C_LFGList.ClearSearchResults()
			-- No group found, creating a new one
			if (foundGroup == false) then
				if (blacklistedApplicationsCount > 0) then
					WorldQuestGroupFinder.prefixedPrint(string.format(L["You have not been applied to %d group(s) because their leader was blacklisted. You can use |c00bfffff/wqgf unbl |c00ffffffto clear the blacklist."], blacklistedApplicationsCount))
				end
				local shortTitle = strsub(title, 0, 31)
				C_LFGList.CreateListing(currentZone, shortTitle, 0, 0, "", "Doing the World Quest "..title.." in "..activityIDs[currentZone]..". Automatically created by WorldQuestGroupFinder. #WQ:"..wqID.."#", WorldQuestGroupFinder.GetConfigValue("autoinvite"))
				WorldQuestGroupFinder.prefixedPrint(string.format(L["A new group finder entry has been created for |c00bfffff%s|c00ffffff in |c00bfffff%s|c00ffffff."], title, activityIDs[currentZone]))
				currentWQ = wqID
				tempWQ = nil
			else 
				WorldQuestGroupFinder.prefixedPrint(string.format(L["You have been applied to |c00bfffff%d|c00ffffff group(s) for the World Quest |c00bfffff%s|c00ffffff."], applicationsCount, title))
			end
		end)
	else
		WorldQuestGroupFinder.prefixedPrint(L["You are not in the right location for this world quest."])
	end
end

StaticPopupDialogs["WORLD_QUEST_FINISHED_LEADER_PROMPT"] = {
  text = L["You have completed the world quest. Would you like to leave the group or unlist it from the Group Finder?"],
  button1 = L["Leave"],
  button2 = L["Stay"],
  button3 = L["Unlist"],
  OnAccept = function()
      LeaveParty()
  end,
  OnAlt = function()
      C_LFGList.RemoveListing()
  end,
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
  preferredIndex = 3,  
}

StaticPopupDialogs["WORLD_QUEST_FINISHED_PROMPT"] = {
  text = L["You have completed the world quest. Would you like to leave the group?"],
  button1 = "Leave",
  button2 = "Stay",  
  OnAccept = function()
      LeaveParty()
  end,
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
  preferredIndex = 3,  
}

StaticPopupDialogs["NEW_WORLD_QUEST_PROMPT"] = {
  text = L["You are currently grouped for another World Quest. Are you sure to want to start another one?"],
  button1 = "Yes",
  button2 = "Cancel",  
  OnAccept = function()
      LeaveParty()
	  C_Timer.After(1, function()
		WorldQuestGroupFinder.InitWQGroup(tempWQ)
	  end)
  end,
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
  enterClicksFirstButton = true,
  preferredIndex = 3,  
}

function WorldQuestGroupFinder.SendWQCompletionPartyNotification (questID)
	local title, factionID, tagID, tagName, worldQuestType, rarity, elite, tradeskillLineIndex = WorldQuestGroupFinder.GetQuestInfo(questID)
	SendChatMessage("[WQGF] World Quest \""..title.."\" completed! "..L[":)"], "PARTY", "", "");
end

function WorldQuestGroupFinder.ShowLeavePrompt()
	if (UnitIsGroupLeader("player")) then
		StaticPopup_Show ("WORLD_QUEST_FINISHED_LEADER_PROMPT")
	else 
		StaticPopup_Show ("WORLD_QUEST_FINISHED_PROMPT")
	end
end

function WorldQuestGroupFinder.CreateConfigMenu()
	local configPanel = CreateFrame("Frame", "WorldQuestGroupFinderConfigFrame", UIParent)
	configPanel.name = "WorldQuestGroupFinder"
	configPanel.okay = function (self) return end
	configPanel.cancel = function (self) return end

	local addonName, addonTitle, addonNotes = GetAddOnInfo('WorldQuestGroupFinder')
	local configPanelText = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	configPanelText:SetPoint('TOPLEFT', 16, -16)
	configPanelText:SetText(addonTitle .. " (" .. L["Brought to you by Robou, EU-Hyjal"] ..")")

	local configPanelDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')

	configPanelDesc:SetPoint('TOPLEFT', configPanelText, 'BOTTOMLEFT', 0, -8)
	configPanelDesc:SetPoint('RIGHT', configPanel, -32, 0)
	configPanelDesc:SetNonSpaceWrap(true)
	configPanelDesc:SetJustifyH('LEFT')
	configPanelDesc:SetJustifyV('TOP')
	configPanelDesc:SetText(addonNotes)
	
	local autoInviteDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	autoInviteDesc:SetPoint('TOPLEFT', configPanelDesc, 'BOTTOMLEFT', -2, -8)
	autoInviteDesc:SetText(L["Auto-invite"])

	local autoInvite = WorldQuestGroupFinder.CreateCheckButton(L["Enable auto-invite"], configPanel, L["Enable auto-invite"], "autoinvite", 'InterfaceOptionsCheckButtonTemplate')
	autoInvite:SetPoint('TOPLEFT', autoInviteDesc, 'BOTTOMLEFT', 0, -2)

	local askToLeaveDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	askToLeaveDesc:SetPoint('TOPLEFT', autoInvite, 'BOTTOMLEFT', 0, -8)
	askToLeaveDesc:SetText(L["Show a dialog to leave the group when the world quest is completed"])

	local askToLeave = WorldQuestGroupFinder.CreateCheckButton(L["Enable group leave dialog"], configPanel, L["Enable group leave dialog"], "askToLeave", 'InterfaceOptionsCheckButtonTemplate')
	askToLeave:SetPoint('TOPLEFT', askToLeaveDesc, 'BOTTOMLEFT', 0, -2)

	local notifyPartyDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	notifyPartyDesc:SetPoint('TOPLEFT', askToLeave, 'BOTTOMLEFT', 0, -8)
	notifyPartyDesc:SetText(L["Notify the group when the world quest is completed"])

	local notifyParty = WorldQuestGroupFinder.CreateCheckButton(L["Enable party notification"], configPanel, L["Enable party notification"], "notifyParty", 'InterfaceOptionsCheckButtonTemplate')
	notifyParty:SetPoint('TOPLEFT', notifyPartyDesc, 'BOTTOMLEFT', 0, -2)

	local languagesDesc = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	languagesDesc:SetPoint('TOPLEFT', notifyParty, 'BOTTOMLEFT', 0, -8)
	languagesDesc:SetText(L["The groups found by the addon are filtered by realm language according to your selection in the Group Finder interface."])

	local languagesDesc2 = configPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
	languagesDesc2:SetPoint('TOPLEFT', languagesDesc, 'BOTTOMLEFT', 0, -4)
	languagesDesc2:SetText(L["It is recommended to select all languages to get a broader realm selection."])

	InterfaceOptions_AddCategory(configPanel)
end

function WorldQuestGroupFinder.CreateCheckButton(name, parent, tooltipText, configKey, template)
	local button = CreateFrame('CheckButton', parent:GetName() .. name, parent, template)
	_G[button:GetName() .. 'Text']:SetText(name)
	button:SetChecked(WorldQuestGroupFinder.GetConfigValue(configKey))
	button:SetScript('OnClick', function(self)
		if self:GetChecked() then
			WorldQuestGroupFinder.SetConfigValue(configKey, true)
		else 
			WorldQuestGroupFinder.SetConfigValue(configKey, false)
		end
		if ( self.dependentControls ) then
			if ( self:GetChecked() ) then
				for _, control in pairs(self.dependentControls) do
					control:Enable()
				end
			else
				for _, control in pairs(self.dependentControls) do
					control:Disable()
				end
			end
		end
		if ( self.inverseDependentControls ) then
			if ( self:GetChecked() ) then
				for _, control in pairs(self.inverseDependentControls) do
					control:Disable()
				end
			else
				for _, control in pairs(self.inverseDependentControls) do
					control:Enable()
				end
			end
		end

	end)
	button:SetScript('OnEnter', function(self)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:SetText(tooltipText, nil, nil, nil, 1, 1)
		GameTooltip:Show() end)
	button:SetScript('OnLeave', function(self) GameTooltip:Hide() end)

	return button
end

function WorldQuestGroupFinder.GetConfigValue(key)
	return WorldQuestGroupFinderConfig[key]
end

function WorldQuestGroupFinder.SetConfigValue(key, value)
	WorldQuestGroupFinderConfig[key] = value
end

function WorldQuestGroupFinder.handleCMD(msg, editbox)
	if (string.lower(msg) == "config") then
		InterfaceOptionsFrame_OpenToCategory("WorldQuestGroupFinder")
	elseif (string.lower(msg) == "debug") then
		if (currentWQ ~= nil) then
			WorldQuestGroupFinder.prefixedPrint(string.format(L["Current World Quest ID is |c00bfffff%s|c00ffffff."], currentWQ))
		else 
			WorldQuestGroupFinder.prefixedPrint(L["No current World Quest."])
		end
	elseif (string.lower(msg) == "unbl") then
		blacklistedLeaders = {}
		WorldQuestGroupFinder.prefixedPrint(L["The leaders blacklist has been cleared."])
	else
		WorldQuestGroupFinder.help()
	end
end

function WorldQuestGroupFinder.help()
	print(L["|c00bfffffSlash commands (/wqgf):"])
	print(L["|c00bfffff /wqgf config : Open addon configuration"])
	print(L["|c00bfffff /wqgf unbl : Clear the leaders blacklist"])
end

function WorldQuestGroupFinder.prefixedPrint(text)
	print("|c00bfffff[WQGF]|c00ffffff "..text)
end 

DefaultConfig = {
	autoinvite = false,
	askToLeave = true,
	notifyPary = true
}